from django_helpers.helpers.views import render_to_string
from django_helpers.templatetags.static_manger import add_js_file, add_css_file, add_jquery

__author__ = 'ajumell'

class Effects(object):
    sliceDown = "sliceDown"
    sliceDownLeft = "sliceDownLeft"
    sliceUp = "sliceUp"
    sliceUpLeft = "sliceUpLeft"
    sliceUpDown = "sliceUpDown"
    sliceUpDownLeft = "sliceUpDownLeft"
    fold = "fold"
    fade = "fade"
    random = "random"
    slideInRight = "slideInRight"
    slideInLeft = "slideInLeft"
    boxRandom = "boxRandom"
    boxRain = "boxRain"
    boxRainReverse = "boxRainReverse"
    boxRainGrow = "boxRainGrow"
    boxRainGrowReverse = "boxRainGrowReverse"

class Themes(object):
    bar = "bar"
    light = "light"
    dark = "dark"
    default = "default"
    wds = "wds"


class NivoSliderSlide(object):
    def __init__(self, url, caption=None, alternate_text="", link=None, transition=None):
        self.url = url
        self.link = link
        self.alternate_text = alternate_text
        self.caption = caption
        self.transition = transition


def js_bool(param):
    if param is True:
        return "true"
    elif param is False:
        return "false"


class NivoSlider(object):
    width = None
    height = None

    slider_id = None
    effect = 'random'
    slices = 15
    box_cols = 8
    box_rows = 4
    animation_speed = 700
    pause_time = 3000
    start_slide = 0
    direction_nav = True
    control_nav = True
    control_bav_thumbs = False
    pause_on_hover = True
    manual_advance = False
    prev_text = 'Prev'
    next_text = 'Next'
    random_start = False
    images = []
    theme_name = "bar"
    css_class = ""

    def __init__(self):

        add_jquery()
        add_js_file('nivo-slider/js/jquery.nivo.slider.pack.js')
        add_css_file('nivo-slider/css/nivo-slider.css')

        self.theme = self.theme_name

    def add_image(self, url, caption=None, alternate_text="", transition=None):
        self.images.append(NivoSliderSlide(url, caption, alternate_text, transition))

    @property
    def theme(self):
        return self.theme_name

    @theme.setter
    def theme(self, val):
        add_css_file('nivo-slider/css/themes/%s/%s.css' % (val, val))
        self.theme_name = val


    def render(self):
        if not self.slider_id:
            raise Exception("Slider ID is require.")

        class_name = "nivoSlider"
        if self.css_class:
            class_name = class_name + " " + self.css_class

        return render_to_string('nivo-slider/nivo-slider.html', {
            "theme": self.theme_name,
            "width": self.width,
            "height": self.height,
            "class_name": class_name,
            "images": self.images,
            "slider_id": self.slider_id,
            "effect": self.effect,
            "slices": self.slices,
            "box_cols": self.box_cols,
            "box_rows": self.box_rows,
            "animation_speed": self.animation_speed,
            "pause_time": self.pause_time,
            "start_slide": self.start_slide,
            "direction_nav": js_bool(self.direction_nav),
            "control_nav": js_bool(self.control_nav),
            "control_nav_thumbs": js_bool(self.control_bav_thumbs),
            "pause_on_hover": js_bool(self.pause_on_hover),
            "manual_advance": js_bool(self.manual_advance),
            "prev_text": self.prev_text,
            "next_text": self.next_text,
            "random_start": js_bool(self.random_start)
        })

    def __str__(self):
        return self.render()

    def __unicode__(self):
        return self.render()


"""
    This file has the settings for sending sms from send grid add-on
    of heroku. just import * from this file and start sending mails.
"""
from os import environ

def has_send_grid():
    return environ.has_key('SENDGRID_USERNAME') and environ.has_key('SENDGRID_PASSWORD')

if has_send_grid():
    EMAIL_PORT = 587
    EMAIL_HOST = 'smtp.sendgrid.net'
    EMAIL_HOST_USER = environ.get('SENDGRID_USERNAME')
    EMAIL_HOST_PASSWORD = environ.get('SENDGRID_PASSWORD')
    EMAIL_USE_TLS = False


from django.contrib import messages
from django.shortcuts import redirect
from django.http import HttpResponse
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.template import loader, RequestContext as Context
from SIS.Library.helpers import can_delete
from form_addons.preview import Form

def getatt(object, name, default=None):
    val = getattr(object, name, default)
    try:
        return val()
    except Exception:
        return val


def is_multipart(Form):
    for x in Form.base_fields:
        y = Form.base_fields[x]
        if y.widget.needs_multipart_form:
            return True
    return False


def mform_view(request, Form, success_message, heading, obj=None, user_field=None, args=None):
    if args is None: args = {}
    if obj is not None: args['instance'] = obj

    if request.method == "POST":
        if is_multipart(Form):
            form = Form(request.POST, request.FILES, **args)
        else:
            form = Form(request.POST, **args)

        if form.is_valid():
            if user_field is not None:
                ret = form.save(False)
                setattr(ret, user_field, request.user)
                ret.save()
            else:
                ret = form.save()
            messages.success(request, success_message)
            return ret, True
        else:
            form.error_message = "Please correct the errors."
    else:
        if obj is not None:
            form = Form(**args)
        else:
            form = Form()

    form.heading = heading
    template = loader.get_template("generic/form.html")
    context = Context(request)
    context['form'] = form
    html = template.render(context)
    return HttpResponse(html)


def form_view(request, Form, success_message, return_url, heading, obj=None, user_field=None, args=None):
    ret = mform_view(request, Form, success_message, heading, obj, user_field, args)
    try: len(ret)
    except Exception: return ret
    return redirect(return_url)


def delete_view(request, obj, name, attr, return_url):
    if can_delete(request) is False:
        template = loader.get_template("generic/confirm.html")
        context = Context(request)
        msg = 'Do you want to delete %s "%s" ?' % (name, getatt(obj, attr))
        msg = msg + "<br />" + "Every data related to this will also be deleted."
        msg = msg + "<br />" + "Are you sure ?"
        context['message'] = msg
        context['back_url'] = return_url
        html = template.render(context)
        return HttpResponse(html)
    else:
        obj.delete()
        messages.success(request, '%s delete successfully.' % name.capitalize())
        return redirect(return_url)


def preview(request, obj, forms, title=None, buttons=None, start=None, end=None):
    if hasattr(obj, 'count'):
        preview_forms = []
        for o in obj:
            preview_form = [Form(o, form) for form in forms]
            preview_forms.extend(preview_form)
    else:
        preview_forms = [Form(obj, form) for form in forms]
    if start is not None: preview_forms.insert(0, Form(None, start))
    if end is not None: preview_forms.append(Form(None, end))
    template = loader.get_template("preview.html")
    context = Context(request)
    context['forms'] = preview_forms
    context['title'] = title
    context['buttons'] = buttons
    html = template.render(context)
    return HttpResponse(html)


def render_view(request, query, heading, actions=None, coloums=None, group_actions=None,
                title='Objects', action_buttons=None, error_message='There are no data to display.'):
    page = request.GET.get("page", '1')
    #    query = query.select_related()
    paginator = Paginator(query, 50)
    try:
        paged_data = paginator.page(page)
    except PageNotAnInteger:
        paged_data = paginator.page(1)
    except EmptyPage:
        paged_data = paginator.page(paginator.num_pages)

    template = loader.get_template("generic/table.html")
    context = Context(request)

    context["objects"] = paged_data
    context['actions'] = actions
    context['coloums'] = coloums
    context['empty_message'] = error_message
    context['group_actions'] = group_actions
    context['action_buttons'] = action_buttons
    context['heading'] = heading
    context['table_title'] = title
    html = template.render(context)
    return HttpResponse(html)


from django.forms import fields
from django.forms.util import flatatt
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django_helpers.templatetags.static_manger import add_jquery, add_jquery_ui, add_js_file

class Widget:
    attrs = None
    has_js = True

    def build_attrs(self, extra_attrs=None, **kwargs):
        """
        This function has to be updated when the corresponding
        django function changes.
        """
        attrs = dict(self.attrs, **kwargs)
        if extra_attrs:
            attrs.update(extra_attrs)

        if attrs.has_key("id"):
            self.html_id = attrs["id"]
        return attrs


class DateInput(Widget, fields.DateInput):
    def __init__(self, attrs=None, format=None, max_date=None,
                 min_date=None, change_year=None, change_month=None):
        add_jquery()
        add_jquery_ui()
        fields.DateInput.__init__(self, attrs, format)

        self.min_date = min_date
        self.max_date = max_date
        self.change_month = change_month
        self.change_year = change_year

    def render_js(self):
        return render_to_string('xs-forms/js/date-field.js', {
            "max_date": self.max_date,
            "min_date": self.min_date,
            "change_year": self.change_year,
            "change_month": self.change_month,
            "id": self.html_id,
            "date_format": self.format
        })


class MaskedInput(Widget, fields.TextInput):
    def __init__(self, mask, placeholder="_", *args, **kwargs):
        add_jquery()
        add_jquery_ui()
        add_js_file('js/jquery.maskedinput-1.3.min.js')

        fields.TextInput.__init__(self, *args, **kwargs)
        self.mask = mask
        self.placeholder = placeholder

    def render_js(self):
        return render_to_string('xs-forms/js/mask-input.js', {
            "mask": self.mask,
            "id": self.html_id,
            "placeholder": self.placeholder
        })


class SpinnerInput(Widget, fields.TextInput):
    def __init__(self, min_value=None, max_value=None, places=None, prefix="",
                 step=1, largeStep=10, *args, **kwargs):
        # TODO: Add more options from plugin
        add_jquery()
        add_jquery_ui()
        add_js_file('js/jquery-ui-spinner.js')
        fields.TextInput.__init__(self, *args, **kwargs)
        self.min_value = min_value
        self.max_value = max_value
        self.places = places
        self.prefix = prefix
        self.step = step
        self.largeStep = largeStep

    def render(self, name, value, attrs=None):
        self.value = value
        return fields.TextInput.render(self, name, self.prefix + str(value), attrs)

    def render_js(self):
        return render_to_string('xs-forms/js/mask-input.js', {
            "min": self.min_value,
            "max": self.max_value,
            "prefix": self.prefix,
            "places": self.places,
            "value": self.value,
            "step": self.step,
            "largeStep": self.largeStep,
            "id": self.html_id
        })


class RatingWidget(Widget, fields.TextInput):
    def __init__(self, number=5, cancel=False, half=False, size=16, *args, **kwargs):
        # TODO: Add more options from plugin
        add_jquery()
        add_js_file('js/jquery.raty.min.js')
        fields.TextInput.__init__(self, *args, **kwargs)
        self.number = number
        self.cancel = cancel
        self.half = half
        self.size = size

    def render(self, name, value, attrs=None):
        if value is None:
            value = 0

        self.input_type = "hidden"
        hidden = fields.TextInput.render(self, name, value)
        final_attrs = self.build_attrs(attrs)
        self.value = value
        self.name = name
        return mark_safe(u'<div%s></div>' % flatatt(final_attrs)) + hidden

    def render_js(self):
        return render_to_string('xs-forms/js/raty.js', {
            "number": self.number,
            "half": self.half,
            "size": self.size,
            "name": self.name,
            "id": self.html_id,
            "score": self.value,
            "cancel": self.cancel
        })



def remove_breaks(string):
    while string.find('\n') >= 0:
        string = string.replace('\n', '')
    return string


def remove_extra_space(string):
    while string.find('  ') >= 0:
        string = string.replace('  ', ' ')
    return string

def remove_extra_breaks(string):
    lines = string.split('\n')
    new_lines = []
    for line in lines:
        line = line.strip()
        if line != "":
            new_lines.append(line)
    return "\n".join(new_lines)



__author__ = 'ajumell'

import datetime
import gdata.analytics.client
import gdata.sample_util
import gdata.service

from django.conf import settings

LAST_UPDATED = {}
CACHE = {}

CLIENT = None

def get_last_updated(app, id):
    date = LAST_UPDATED.get(app)
    if date is None:
        date = {}
        LAST_UPDATED[app] = date
    return date.get(id)


def set_last_updated(app, id, val):
    date = LAST_UPDATED.get(app)
    if date is None:
        date = {}
        LAST_UPDATED[app] = date
    date[id] = val


def get_cache(app, id):
    date = CACHE.get(app)
    if date is None:
        date = {}
        CACHE[app] = date
    return date[id]


def set_cache(app, id, val):
    date = CACHE.get(app)
    if date is None:
        date = {}
        CACHE[app] = date
    date[id] = val

# TODO: Need more settings options

def get_feed(query):
    global CLIENT
    if CLIENT is None:
        SOURCE_APP_NAME = 'Xeoscript Daily Analytics Application'
        my_client = gdata.analytics.client.AnalyticsClient()
        my_client.client_login(settings.GMAIL_USERNAME, settings.GMAIL_PASSWORD, SOURCE_APP_NAME,
            my_client.auth_service)
        CLIENT = my_client
    return CLIENT.GetDataFeed(query)

def get_metric(app_id, id):
    today = datetime.date.today()
    if today == get_last_updated(app_id, id):
        val = get_cache(app_id, id)
        if val:
            return val
    else:
        set_last_updated(app_id, id, today)

    print 'Collecting info from google...'

    start = datetime.date.today() - datetime.timedelta(years=10)
    data_query = gdata.analytics.client.DataFeedQuery({
        'ids': 'ga:%d' % id,
        'start-date': start,
        'end-date': today,
        'metrics': 'ga:%s' % app_id,
        'sort': 'ga:visits',
        'max-results': '1000'})

    feed = get_feed(data_query)
    val = feed.aggregates.metric[0].value
    set_cache(app_id, id, val)
    return val

def get_page_visits(id):
    app_id = "visits"
    return get_metric(app_id, id)





import re
from django_helpers.helpers.templatetags import parse_args
from django_helpers.helpers.views import render_to_string

__author__ = 'ajumell'

from django import template
from django.template.base import TemplateSyntaxError
from django.template.defaulttags import Node

youtube_regexp = re.compile(r"^(http://)?(www\.)?(youtube\.com/watch)?(.*)(\?v=)?(?P<id>[A-Za-z0-9\-=_]{11})")
register = template.Library()

class YoutubeNode(Node):

    def __init__(self, args, kwargs):
        self.args = args
        self.kwargs = kwargs

        self.video_id = args[0]
        self.width = kwargs.get('width', '800px')
        self.height = kwargs.get('height', "400px")
        self.full_screen = kwargs.get('fullscreen', True)

    def render(self, context):
        url = self.video_id.resolve(context)
        width = self.width.resolve(context)
        height = self.height.resolve(context)

        try:
            full_screen = self.full_screen.resolve(context)
        except Exception:
            full_screen = self.full_screen

        if full_screen:
            full_screen = "allowfullscreen"
        else:
            full_screen = ""

        match = youtube_regexp.match(url)
        if not match:
            return ""
        video_id = match.group('id')
        return render_to_string('tags/video/youtube.html',{
            'video_id': video_id,
            'width': width,
            'height': height,
            'full_screen': full_screen
        })

@register.tag
def youtube(parser, token):
    bits = token.split_contents()
    args, kwargs = parse_args(bits, parser)
    if len(args) < 1:
        raise TemplateSyntaxError("'%s' takes at least one arguments." % bits[0])
    return YoutubeNode(args, kwargs)





__author__ = 'ajumell'

from django.conf import settings
import tweepy

HasAuth = False

def auth():
    global HasAuth

    # TODO: Check auth using tweepy
    if HasAuth:
        return

    consumer_key = getattr(settings, 'TWITTER_CONSUMER_KEY', "")
    consumer_secret = getattr(settings, 'TWITTER_CONSUMER_SECRET', "")

    token = getattr(settings, 'TWITTER_TOKEN', "")
    secret = getattr(settings, 'TWITTER_SECRET', "")

    # TODO: Validation

    auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
    auth.set_access_token(token, secret)

    tweepy.API(auth)

    HasAuth = True



def get_public_tweets(username):
    return tweepy.api.user_timeline(username)



def send_direct_message(username, contents):
    auth()
    tweepy.api.send_direct_message(screen_name=username, text=contents)

def tweet(contents):
    auth()
    tweepy.api.update_status(contents)

def tweet_page(link):
    auth()
    # TODO: Complete



from django.template import TemplateSyntaxError, Node
from django.template.defaulttags import register, CsrfTokenNode
from django_helpers.apps.form_renderer import FormRenderer
from django_helpers.helpers.templatetags import parse_args

__author__ = 'ajumell'

class FormRendererNode(Node):
    def __init__(self, form, form_id, template, kwargs=None):
        self.form = form
        self.form_id = form_id
        self.template = template
        self.kwargs = kwargs

    def render(self, context):
        csrf_token = CsrfTokenNode().render(context)
        template = self.template
        form_id = self.form_id.resolve(context)
        form = self.form.resolve(context)

        obj = FormRenderer(form, form_id=form_id, csrf_token=csrf_token, template=template)
        for arg, value in self.kwargs.items():
            value = value.resolve(context)
            if hasattr(obj, arg):
                setattr(obj, arg, value)
        return obj.render()


@register.tag
def render_form(parser, token):
    bits = token.split_contents()
    if len(bits) < 3:
        raise TemplateSyntaxError("'%s' needs at least two arguments." % bits[0])
    args, kwargs = parse_args(bits, parser)
    template = None

    form = args[0]
    form_id = args[1]
    if len(args) > 2:
        template = bits[3]

    return FormRendererNode(form, form_id, template, kwargs)


from django.core.urlresolvers import reverse
from django.forms import fields
from django.template.loader import render_to_string
from django_helpers.forms.widgets import Widget
from django_helpers.apps.autocomplete import register

class AutoCompleteWidget(Widget, fields.TextInput):
    formatted_value = None
    template = 'autocomplete/js/auto-complete.js'

    def __init__(self, attrs=None, delay=0, min_length=1, lookup=None):
        fields.TextInput.__init__(self, attrs)
        register(lookup)
        self.auto_complete = lookup()
        self.min_length = min_length
        self.delay = delay

    def get_formatted_value(self, value):
        if not value:
            return ""

        # Edit mode in model field.
        if self.formatted_value is None:
            auto_complete = self.auto_complete
            instance = auto_complete.get_instance(value)
            val = auto_complete.format_value(instance)
        else:
            val = self.formatted_value
        self.formatted_value = val
        self.value = value
        return val

    def render(self, name, value, attrs=None):
        if value is None:
            value = ""
        self.value = value

        self.input_type = "hidden"
        hidden = fields.TextInput.render(self, name, value)
        self.name = name
        self.input_type = "text"
        value = self.get_formatted_value(value)
        display = fields.TextInput.render(self, name + "_display", value, attrs)
        return display + hidden

    def id_for_label(self, id_):
        return '%s_display' % id_

    def render_js(self):
        source = reverse(self.auto_complete.name)
        value = self.value or ""
        op = render_to_string(self.template, {
            "min_length": self.min_length,
            "delay": self.delay,
            "source": source,
            "id": self.html_id,
            "name": self.name,
            "label": self.formatted_value,
            "value": value
        })
        return op


class SimpleAutoCompleteWidget(AutoCompleteWidget):
    template = 'autocomplete/js/auto-complete-simple.js'

    def get_formatted_value(self, value):
        return value


class TokenInputWidget(Widget, fields.TextInput):
    instances = None

    def __init__(self, lookup, hint_text=None, no_result_text=None, searching_text=None, delay=0,
                 min_chars=1, limit=None, prevent_duplicates=True, theme=None, *args, **kwargs):
        fields.TextInput.__init__(self, *args, **kwargs)
        register(lookup)
        self.auto_complete = lookup()
        self.hint_text = hint_text
        self.no_result_text = no_result_text
        self.searching_text = searching_text
        self.delay = delay
        self.min_chars = min_chars
        self.limit = limit
        self.prevent_duplicates = prevent_duplicates
        self.theme = theme

    def render(self, name, value, attrs=None):
        self.name = name
        if self.instances is None:
            auto_complete = self.auto_complete
            self.instances = auto_complete.get_instances(value)
        return fields.TextInput.render(self, name, "", attrs)


    def render_js(self):
        source = reverse(self.auto_complete.name)
        instances = self.instances
        formatted_values = []
        if instances:
            auto_complete = self.auto_complete
            for instance in instances:
                formatted_values.append({
                    "name": auto_complete.format_value(instance),
                    "id": instance.pk
                })

        op = render_to_string('autocomplete/js/token-input.js', {
            "hint_text": self.hint_text,
            "no_result_text": self.no_result_text,
            "searching_text": self.searching_text,
            "delay": self.delay,
            "min_chars": self.min_chars,
            "limit": self.limit,
            "theme": self.theme,
            "prevent_duplicates": self.prevent_duplicates,
            "source": source,
            "id": self.html_id,
            "name": self.name,
            "values": formatted_values
        })
        return op


"""
This file will help to generate settings for a
heroku based django application easily. import *
from this file will automatically generate database
and email settings for heroku.


Database Settings
=================
Database settings will be parsed from the
herok config files. There are many solutions
for databases in heroku. They will be used as
given below.

Order
-----
1. Clear DB Database
2. Heroku Shared PG Database

Email Settings
==============
Currently send grid settings will be generated
automatically.

TODO:
Need more settings.

"""
__author__ = 'ajumell'
from send_grid_settings import has_send_grid
from database import has_clear_db
from django_helpers.helpers.settings.gmail_settings import has_gmail



if has_clear_db():
    from clear_db_settings import DATABASES
else:
    pass

if has_send_grid():
    from send_grid_settings import  EMAIL_HOST, EMAIL_HOST_PASSWORD, EMAIL_HOST_USER, EMAIL_PORT, EMAIL_USE_TLS
elif has_gmail():
    from django_helpers.helpers.settings.gmail_settings import EMAIL_HOST, EMAIL_HOST_PASSWORD, EMAIL_HOST_USER,EMAIL_PORT, EMAIL_USE_TLS



# http://djangosnippets.org/snippets/539/
# From django snippets

from django import template

class AssignNode(template.Node):
    def __init__(self, name, value):
        self.name = name
        self.value = value

    def render(self, context):
        context[self.name] = self.value.resolve(context, True)
        return ''


def do_assign(parser, token):
    """
    Assign an expression to a variable in the current context.

    Syntax::
        {% assign [name] [value] %}
    Example::
        {% assign list entry.get_related %}

    """
    bits = token.contents.split()
    if len(bits) != 3:
        raise template.TemplateSyntaxError("'%s' tag takes two arguments" % bits[0])
    value = parser.compile_filter(bits[2])
    return AssignNode(bits[1], value)

register = template.Library()
register.tag('assign', do_assign)


import os
import urlparse

__author__ = 'ajumell'

# Register database schemes in URLs.
urlparse.uses_netloc.append('mysql')

def has_clear_db():
    """
    This function returns true if the clear db database url is present
    in the environmental variable. The environmental variable will be
    automatically added by heroku if clear db addon is used in the project.

    @return: True if the clear db database url is present in the
    enviroment variables.
    """
    return 'CLEARDB_DATABASE_URL' in os.environ


def parse_clear_db(throw_exception=False, print_exception=False):
    """
    This function returns a dictionary with clear db database settings.

    The database settings are parased from the clear db database url enviroment
    variable. This will be added by heroku when clear db addon is used in the
    project.

    @param throw_exception: Throws the exception if any exception occures.
    @param print_exception: Prints the exception data if any exception occures.
    @return: A dictionary with the clear db database settings.
    """
    try:
        if has_clear_db():
            config = {}
            url = urlparse.urlparse(os.environ['CLEARDB_DATABASE_URL'])

            # Ensure default database exists.

            DB_NAME = url.path[1:]
            i = DB_NAME.find('?')
            if i > 0:
                DB_NAME = DB_NAME[:i]

            # Update with environment configuration.
            config.update({
                'NAME': DB_NAME,
                'USER': url.username,
                'PASSWORD': url.password,
                'HOST': url.hostname,
                'PORT': url.port,
                })

            if url.scheme == 'mysql':
                config['ENGINE'] = 'django.db.backends.mysql'
            return config
        else:
            raise Exception("Clear DB addon not installed.")
    except Exception, data:
        if print_exception:
            print data
        if throw_exception:
            raise
        return None



from django.utils.simplejson import dumps
import time
from urls import  urlpatterns
from django.db.models import Q
from django.conf.urls import url
from django.http import HttpResponse

from django_helpers.templatetags.static_manger import add_jquery, add_jquery_ui


lookups = dict()

class AutoComplete(object):
    query_set = None
    search_fields = []
    url_search_parameters = None
    requires_auth = False

    __name__ = "__undefined__"

    def search_term(self, term):
        query = None
        queryset = self.query_set
        fields = self.search_fields
        for field in fields:
            kwargs = {field + "__contains": term}
            if query is None:
                query = Q(**kwargs)
            else:
                query = query | Q(**kwargs)
        return queryset.filter(query)

    def search_url_paramters(self, query, url_params):
        url_search_parameters = self.url_search_parameters
        if url_search_parameters is not None:
            for parameter in url_search_parameters:
                args_dict = {parameter: url_params.get(parameter, '')}
                query = query.filter(**args_dict)
        return query

    def search(self, term, url_params):
        query = self.search_term(term)
        return self.search_url_paramters(query, url_params)

    def get_instances(self, ids):
        return self.query_set.filter(pk__in=ids)

    def get_instance(self, id):
        return self.query_set.get(pk=id)

    def simple_format(self, instance, string, *fields):
        lst = []
        for term in fields:
            lst.append(getattr(instance, term))
        return string % tuple(lst)

    def format_value(self, instance):
        return str(instance)

    def get_search_term(self, request):
        term = request.GET.get("term")
        if term is None:
            term = request.GET.get("q")
        return term

    def get_results(self, request, params):
        term = self.get_search_term(request)
        query = self.search(term, params)
        results = []
        for result in query:
            val = self.format_value(result)
            # Renamed value for adding support for
            # jQuery token field. The value can be
            # controlled for jQuery auto complete.
            results.append({
                "id": result.id,
                "label": val,
            })
        return results


def autocomplete_lookup(request, lookup, **kwargs):
    # If lookup does not exists then return blank array.
    # This is prevent any exceptions at runtime.
    if not lookups.has_key(lookup):
        return HttpResponse("[]")

    auto_complete = lookups[lookup]()

    results = auto_complete.get_results(request, kwargs)
    return HttpResponse(dumps(results))


def create_reg(name):
    return "(?P<%s>.*)/" % name

url_prefix = 'auto-complete-'

def register(autocomplete_class):
    add_jquery()
    add_jquery_ui()

    if isinstance(autocomplete_class, AutoComplete):
        raise Exception("AutoComplete class is required not instance.")

    if not hasattr(autocomplete_class, 'name'):
        name = str(time.time())
        name = name.replace('.', '')
    else:
        name = getattr(autocomplete_class, 'name')

    if not name.startswith(url_prefix):
        name = url_prefix + name
    setattr(autocomplete_class, 'name', name)

    lookups[name] = autocomplete_class
    reg = "%s/" % name
    extra_url_parameters = autocomplete_class.url_search_parameters
    if extra_url_parameters is not None:
        for parameter in extra_url_parameters:
            reg += create_reg(parameter)
    reg += "$"

    pattern = url(r"%s" % reg, autocomplete_lookup, name=name, kwargs={
        "lookup": name
    })
    urlpatterns.append(pattern)



from django import forms
from django.utils.translation import ugettext_lazy as _
from widgets import DateInput
from django_helpers.apps.autocomplete import fields
from django_helpers.apps.autocomplete.widgets import AutoCompleteWidget, SimpleAutoCompleteWidget

def set_property(field, args, name, target):
    if name in args:
        val = args[name]
        setattr(field, target, val)


def update_class_name(field, class_name):
    if class_name is not None:
        attr = field.widget.attrs
        old_class = attr.get('class', '')
        attr['class'] = class_name if old_class == '' else "%s %s" % (old_class, class_name)


def factory(field, help_text=None, required=None,
            label=None, error_messages=None, initial_value=None,
            class_name=None, container_class=None, label_class=None,
            class_names=None, extra_options=None, widget_options=None
):
    widget = field.widget

    if label is not None:
        field.label = _(label)

    if container_class is not None:
        field.container_class = container_class

    if label_class is not None:
        field.label_class = label_class

    if help_text is not None:
        field.help_text = help_text

    if class_name is not None:
        update_class_name(field, class_name)

    if error_messages is not None:
        field.error_messages.update(error_messages)

    if required is not None:
        field.required = required

    if initial_value is not None:
        field.initial = initial_value

    if extra_options is not None:
        #print extra_options.values()
        for key, value in extra_options.items():
            setattr(field, key, value)

    if class_names is not None:
        for class_name in class_names:
            update_class_name(field, class_name)

    if widget_options is not None:
        for key, value in widget_options.items():
            setattr(widget, key, value)

    field.localize = False
    field.required = required

    return field


def CharField(label=None, max_length=None, min_length=None,
              required=None, class_names=None, class_name=None):
    return factory(
        forms.CharField(),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names,
        extra_options={
            "max_length": max_length,
            "min_length": min_length,
            }
    )


def PasswordField(label=None, max_length=None, min_length=None,
                  required=None, class_names=None, class_name=None):
    return factory(
        forms.CharField(widget=forms.PasswordInput()),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names,
        extra_options={
            "max_length": max_length,
            "min_length": min_length,
            }
    )


def EmailField(label=None, max_length=None, min_length=None,
               required=None, class_names=None, class_name=None):
    return factory(
        forms.EmailField(),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names,
        extra_options={
            "max_length": max_length,
            "min_length": min_length,
            "email": True
        }
    )


def IntegerField(label=None, max_value=None, min_value=None,
                 required=None, class_names=None, class_name=None, initial_value=None):
    if class_names is None:
        class_names = []
    class_names.append('spin')
    return factory(
        forms.IntegerField(min_value=min_value, max_value=max_value),
        initial_value=initial_value,
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names,
        extra_options={
            "integer": True
        }
    )


def DateField(label=None, format=None, max_date=None,
              min_date=None, change_year=None, change_month=None,
              required=None, class_names=None, class_name=None):
    return factory(
        forms.DateField(widget=DateInput(
            max_date=max_date,
            min_date=min_date,
            format=format,
            change_year=change_year,
            change_month=change_month
        )),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names
    )


def AutoCompleteField(lookup, label=None, min_length=2, delay=0,
                      required=None, class_names=None, class_name=None):
    widget = AutoCompleteWidget(delay=delay, min_length=min_length,
                                lookup=lookup)
    return factory(
        fields.AutoCompleteField(lookup=lookup, widget=widget),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names
    )


def SimpleAutoCompleteField(lookup, label=None, min_length=2, delay=0,
                            required=None, class_names=None, class_name=None):
    widget = SimpleAutoCompleteWidget(delay=delay, min_length=min_length,
                                      lookup=lookup)
    return factory(
        fields.SimpleAutoCompleteField(widget=widget),
        label=label,
        required=required,
        class_name=class_name,
        class_names=class_names
    )



from django.http import HttpResponse
from django.db.models import Q
from django.utils.simplejson import dumps

def data_table(request, query, fields, extra_params=None, **kwargs):
    """
        A view to provide data to jQuery data table plugin. This plugin will
    automatically collect parameters send by the data table plugin and
    returns necessary data as json object. This function can be directly
    called from a urls file.

        The parameters required for this function are request, query and fields.

        Request is the http request send by the plugin. It contains the parameters
    for the data to be responded. This function will automatically collect the
    informations from the request.

        Query is the model query from which the filters has to be applied. It is
    given as query so that custom filters can be applied.

        Fileds are the fileds that has to be send as response in correct order. The
    data send by this function is array and not object so order is very important.

    Example 1
    data_table(request, User.objects.all(), ["username", "first_name", "last_name"])

    Example 2
    url(r'^students/ajax/$', data_table, kwargs={
        "query": Student.objects.all(),
        "fields" : [
            "id",
            "first_name",
            "last_name",
            "college__name",
            "course__name"
        ]
    }, name='students-data-table')


    @param request: A HttpRequest object
    @type request: django.http.HttpRequest

    @param fields: A array of fields to be returned as response
    @type fields: list

    @param query: Query for which filters has to be applied.
    @type query: django.db.models.query.QuerySet

    """
    # TODO: Latest docs of url params


    # Perform extra filtering from named url parameters
    # send from url dispatcher.
    if extra_params is not None:
        filter_dict = {}
        for param_name in extra_params:
            param_value = kwargs.get(param_name, '')
            if param_value != '':
                filter_dict[param_name] = param_value
        query = query.filter(**filter_dict)

    GET = request.GET
    gt = GET.get

    no_of_coloums = int(gt('iColumns'))
    echo = gt('sEcho')
    start_pos = int(gt('iDisplayStart'))
    max_items = int(gt('iDisplayLength'))
    search_term = gt('sSearch')
    total_length = query.count()
    need_related = False

    # Collect sort fields and apply sorts
    sorts, i = [], 0
    while i < no_of_coloums:
        # Detect if select_related is needed.
        try:
            if not need_related and fields[i].find('__') > -1:
                need_related = True
        except Exception:
            pass

        col = gt("iSortCol_%d" % i)
        if col is not None:
            col = int(col)
            field = fields[col]
            if gt("sSortDir_%d" % i) == "desc":
                field = "-" + field
            sorts.append(field)
        i += 1
    sorts = tuple(sorts)
    query = query.order_by(*sorts)

    # Select related fields if necessary
    if need_related:
        query = query.select_related()

    if search_term != "":
        # Collect sort fields and apply search
        searches, i = [], 0
        while i < no_of_coloums:
            if gt("bSearchable_%d" % i) == "true":
                field = fields[i] + "__contains"
                kwargs = {field: search_term}
                searches.append(Q(**kwargs))
            i += 1
        searches = tuple(searches)

        # Search
        i, q, search_length = 1, searches[0], len(searches)
        while i < search_length:
            q = q | searches[i]
            i += 1
        query = query.filter(q)

    current_length = query.count()
    query = query[start_pos:start_pos + max_items]
    query = query.values_list(*fields)
    result = {
        "aaData": list(query),
        "sEcho": echo,
        "iTotalRecords": total_length,
        "iTotalDisplayRecords": current_length,
        }
    return HttpResponse(dumps(result))



from django import forms
from widgets import AutoCompleteWidget

class AutoCompleteField(forms.Field):
    widget = AutoCompleteWidget

    def __init__(self, lookup, required=True, widget=None, label=None, initial=None,
                 help_text=None, error_messages=None, show_hidden_initial=False,
                 validators=None, localize=False):
        if validators is None:
            validators = []
        if widget is None:
            widget = self.widget(lookup=lookup)
        forms.Field.__init__(self, required, widget, label, initial,
            help_text, error_messages, show_hidden_initial,
            validators, localize)
        self.auto_complete = lookup()

    def clean(self, value):
        value = forms.Field.clean(self, value)
        auto_complete = self.auto_complete
        try:
            if not value: raise
            instance = auto_complete.get_instance(value)
            self.widget.formatted_value = auto_complete.format_value(instance)
            return instance
        except Exception:
            self.widget.formatted_value = ""
            if self.required:
                raise forms.ValidationError(self.error_messages['required'])
            return None


class SimpleAutoCompleteField(forms.Field):
    widget = AutoCompleteWidget

    def clean(self, value):
        value = forms.Field.clean(self, value)
        self.widget.formatted_value = value
        return value


class ManyToManyAutoCompleteField(forms.Field):
    def __init__(self, lookup, *args, **kwargs):
        forms.Field.__init__(self, *args, **kwargs)
        self.auto_complete = lookup()

    def clean(self, value):
        value = forms.Field.clean(self, value)
        try:
            if not value:
                raise
            value = value.split(',')
            auto_complete = self.auto_complete
            instances = auto_complete.get_instances(value)
            self.widget.instances = instances
            return instances
        except Exception:
            self.widget.formatted_value = ""
            if self.required:
                raise forms.ValidationError(self.error_messages['required'])
            return None


import time

from django.core.urlresolvers import reverse
from urls import  urlpatterns
from django.db.models import Q
from django.conf.urls import url
from django.http import HttpResponse
from django.utils.simplejson import dumps
from django.template import Template, loader, RequestContext, Context

from django_helpers.templatetags.static_manger import add_css_file, add_js_file, add_jquery

__author__ = "ajumell"

tables = dict()

class DataTableColoum(object):
    def __init__(self, field, title, searchable, sortable):
        self.field = field
        self.title = title
        self.searchable = searchable
        self.sortable = sortable

    def get_html(self, request, object):
        value = getattr(object, self.field)
        return value


    def need_related(self):
        return self.field.find('__') > -1


class DataTableClientRenderedColoum(DataTableColoum):
    """
    This class is still under planning.
    """
    pass


class DataTableTemplateColoum(DataTableColoum):
    """
    A Data table column which will return a rendered
    a template instead of a database field.
    """

    def __init__(self, field, title, searchable, sortable, template,
                 load_template=True, width=None, value_name='value'):
        """
        @param field: The name of the field to be passed to the template.
        @param title: Title of the column which will be displayed in the table header.
        @param searchable: jQuery data table option searchable
        @param sortable: jQuery data table option sortable
        @param template: Template to be rendered as the return value.
        This can be either a django.template.Template object or a sting.
        If a string is passed then that template has to be loaded by give
        load_template parameter as True.
        @param load_template: If this parameter is true the template will
        be loaded.
        @param width: Width of the column
        @param value_name: name of the variable that has to be passed to the template.
        Its default value is "value"
        """
        DataTableColoum.__init__(self, field, title, searchable, sortable)
        if load_template:
            template = loader.get_template(template)
        self.template = template
        self.width = width
        self.value_name = value_name

    def get_html(self, request, object):
        value = getattr(object, self.field)
        template = self.template

        if not isinstance(template, Template):
            template = Template(self.template)

        context = RequestContext(request, {
            self.value_name: value,
            "width": self.width
        })

        return template.render(context)


class DataTable(object):
    query = None
    url_search_parameters = None

    width = None
    class_name = ""
    name = None
    coloums = []
    table_id = None
    display_length = 10
    ajax_source = ''
    loading_message = 'Please wait while data is loading'
    ui_themes = False

    info_empty = None
    info_loading = None
    info_processing = None

    dom = '<f>t<plir>'
    template = 'data_tables/table.html'

    def __init__(self):
        add_css_file('css/table.css')
        add_css_file('css/ui.css')
        add_jquery()
        add_js_file('js/jquery.dataTables.js')

    def __unicode__(self):
        template = loader.get_template(self.template)
        context = Context({
            'display_length': self.display_length,
            'dom': self.dom,
            'ajax_source': reverse(self.table_id),
            'columns': self.coloums,
            "table_id": self.table_id,
            "ui_themes": self.ui_themes,
            'width': self.width,
            'loading_message': self.loading_message,

            'info_loading': self.info_loading,
            'info_processing': self.info_processing,
            'info_empty': self.info_empty
        })
        return template.render(context)

    def get_data(self, request, kwargs):
        GET = request.GET
        gt = GET.get

        columns = self.coloums
        query = self.query.filter()
        extra_params = self.url_search_parameters
        if extra_params is not None:
            filter_dict = {}
            for param_name in extra_params:
                param_value = kwargs.get(param_name, '')
                if param_value != '':
                    filter_dict[param_name] = param_value
            query = query.filter(**filter_dict)

        echo = gt('sEcho')
        start_pos = int(gt('iDisplayStart'))
        max_items = int(gt('iDisplayLength'))
        search_term = gt('sSearch')

        total_length = query.count()
        need_related = False

        fields, sortings, search, i = [], [], None, -1
        for column in columns:
            fields.append(column.field)
            i += 1
            need_related = need_related or column.need_related()

            if column.searchable and search_term != "":
                field = column.field + "__contains"
                kwargs = {field: search_term}
                if search is None:
                    search = Q(**kwargs)
                else:
                    search = search | Q(**kwargs)

        # Sorting
        total_sort_columns = gt('iSortingCols')
        total_sort_columns = int(total_sort_columns)
        while total_sort_columns:
            total_sort_columns -= 1
            column_number = int(gt('iSortCol_%d' % total_sort_columns))
            sort_order = gt('sSortDir_%d' % total_sort_columns)
            column = columns[column_number]
            if column.sortable:
                field = column.field
                print sort_order
                if sort_order == 'desc':
                    field = '-' + field
                sortings.append(field)


        fields = tuple(fields)
        if need_related:
            query = query.select_related()
        query = query.order_by(*sortings)
        if search is not None:
            query = query.filter(search)
        query = query.only(*fields)

        current_length = query.count()
        query = query[start_pos:start_pos + max_items]
        datas = []

        for record in query:
            data = []
            for column in columns:
                data.append(column.get_html(request, record))
            datas.append(data)

        result = {
            "aaData": datas,
            "sEcho": echo,
            "iTotalRecords": total_length,
            "iTotalDisplayRecords": current_length,
        }
        return HttpResponse(dumps(result))


def get_data(request, name, **kwargs):
    if not tables.has_key(name):
        return HttpResponse()

    table = tables[name]

    results = table.get_data(request, kwargs)
    return results


def create_reg(name):
    return "(?P<%s>.*)/" % name

url_prefix = 'data-table-'

def register(data_table):
    if isinstance(data_table, DataTable):
        raise Exception("DataTable class is required not instance.")

    if not issubclass(data_table, DataTable):
        raise Exception("A Sub class of DataTable is required.")

    name = getattr(data_table, 'table_id', None)
    if not name:
        name = str(time.time())
        name = name.replace('.', '')


    if not name.startswith(url_prefix):
        name = url_prefix + name

    setattr(data_table, 'table_id', name)
    data_table = data_table()
    tables[name] = data_table
    reg = "%s/" % name
    extra_url_parameters = data_table.url_search_parameters
    if extra_url_parameters is not None:
        for parameter in extra_url_parameters:
            reg += create_reg(parameter)
    reg += "$"

    pattern = url(r"%s" % reg, get_data, name=name, kwargs={
        "name": name
    })

    urlpatterns.append(pattern)
    return data_table



from django.template import Node, TemplateSyntaxError
from django_helpers.helpers.templatetags import parse_args
from django_helpers.helpers.views import render_to_string
from django.template.defaulttags import register

__author__ = 'ajumell'

class GoogleAnalyticsNode(Node):
    def __init__(self, code):
        self.code = code

    def render(self, context):
        code = self.code.resolve(context)
        return render_to_string('google-analytics.html',{
            'CODE': code
        })

@register.tag
def analytics_js(parser, token):
    bits = token.split_contents()
    if len(bits) != 2:
        raise TemplateSyntaxError("'%s' needs at least one arguments." % bits[0])
    args, kwargs = parse_args(bits, parser)
    return GoogleAnalyticsNode(*args, **kwargs)



from django_helpers.views.helpers import render_template

__author__ = 'ajumell'

class CheckJsMiddleware:
    def process_request(self, request):
        """
        Checks for cookie has js cookie
        @param request:
        @type request:django.http.HttpRequest
        @return:
        """
        cookie = request.COOKIES.get('has_js', None)
        if cookie is None:
            data = {
                'location': request.get_full_path()
            }
            return render_template(request, 'check_js.html',data)


"""
    This file has the settings for sending email from gmail.
    just import * from this file and start sending mails.

    Requirements:
    GMAIL_USERNAME Environmental Variable
    GMAIL_PASSWORD Environmental Variable

    Usage:
    from django_helpers.helpers.settings.gmail_settings import *
"""

__author__ = 'ajumell'

from os import environ

def has_gmail():
    return environ.has_key('GMAIL_USERNAME') and environ.has_key('GMAIL_PASSWORD')

if has_gmail():
    EMAIL_PORT = 587
    EMAIL_USE_TLS = True
    EMAIL_HOST = 'smtp.gmail.com'
    EMAIL_HOST_USER = environ.get('GMAIL_USERNAME')
    EMAIL_HOST_PASSWORD = environ.get('GMAIL_PASSWORD')


from django.core.cache import cache

__author__ = 'ajumell'

import httplib

from urlparse import urlparse
from django.core.files.storage import Storage
from django.conf import settings
from flickrapi import FlickrAPI, IllegalArgumentException
from flickrapi.multipart import Part, Multipart
from flickrapi import make_utf8

IMAGE_TYPES = {
    'square': 'Square',
    'thumbnail': 'Thumbnail',
    'small': 'Small',
    'medium': 'Medium',
    'large': 'Large'
}

IMAGE_TYPES_PRIORITY = ['Square', 'Thumbnail', 'Small', 'Medium', 'Large']

class FlickrAPIHack(FlickrAPI):
    def __init__(self, api_key, secret=None, username=None,
                 token=None, format='etree', store_token=True, cache=False):
        FlickrAPI.__init__(self, api_key, secret, username, token, format, store_token, cache)

    def _FlickrAPI__upload_to_form(self, form_url, filename, file_obj, callback, **kwargs):
        """Can upload a file using file object"""

        if not filename:
            raise IllegalArgumentException("filename must be specified")
        if not self.token_cache.token:
            raise IllegalArgumentException("Authentication is required")

        format = self._FlickrAPI__extract_upload_response_format(kwargs)

        # Update the arguments with the ones the user won't have to supply
        arguments = {
            'auth_token': self.token_cache.token,
            'api_key': self.api_key
        }

        arguments.update(kwargs)

        # Convert to UTF-8 if an argument is an Unicode string
        kwargs = make_utf8(arguments)

        if self.secret:
            kwargs["api_sig"] = self.sign(kwargs)
        url = "http://%s%s" % (FlickrAPI.flickr_host, form_url)

        # construct POST data
        body = Multipart()

        for arg, value in kwargs.iteritems():
            part = Part({'name': arg}, value)
            body.attach(part)

        content = file_obj.read()
        file_obj.close()

        filepart = Part({'name': 'photo', 'filename': filename}, content, 'image/jpeg')
        body.attach(filepart)

        return self._FlickrAPI__wrap_in_parser(self._FlickrAPI__send_multipart, format,
            url, body, callback)

    def upload(self, filename, callback=None, **kwargs):
        raise Exception("Use upload_file method.")


    def upload_file(self, filename, file_obj, callback=None, **kwargs):
        return self._FlickrAPI__upload_to_form(FlickrAPI.flickr_upload_form, filename, file_obj, callback, **kwargs)


class FlickrStorageException(Exception):
    pass


class FlickrStorage(Storage):
    def __init__(self, options=None):
        self.options = {
            'cache': True,
        }

        self.options.update(options or settings.FLICKR_STORAGE_OPTIONS)
        self.flickr = FlickrAPIHack(self.options['api_key'],
            self.options['api_secret'],
            self.options['username'],
            self.options['token'],
            cache=self.options['cache'])

        if self.options['cache']:
            self.flickr.cache = cache

        (self.token, frob) = self.flickr.get_token_part_one(perms='delete')
        #TODO: move to management command
        if not self.token:
            raw_input('Press Enter...')

        self.flickr.get_token_part_two((self.token, frob))

    def _check_response(self, resp):
        if resp.attrib['stat'] != 'ok':
            err = resp.find('err')
            raise FlickrStorageException, "Error %s: %s" % (err.attrib['code'], err.attrib['msg'])

    def delete(self, name):
        resp = self.flickr.photos_delete(photo_id=name)

    def size(self, name):
        url = self.url(name)
        u = urlparse(url)
        conn = httplib.HTTPConnection(u.hostname)
        conn.request('HEAD', u.path)
        resp = conn.getresponse()
        return int(resp.getheader('content-length'))

    def _save(self, name, content):
        content.seek(0)             #ImageField read first 1024 bytes
        name = name.encode('utf-8')
        resp = self.flickr.upload_file(name, content.file)
        self._check_response(resp)
        name = resp.find('photoid').text
        content.close()
        return name

    def exists(self, name):
        return False

    def url(self, name, img_type=None):
        resp = self.flickr.photos_getSizes(photo_id=name)
        self._check_response(resp)
        value = ""
        prev_rank = -1
        for size in resp.findall('sizes/size'):
            label = size.attrib['label']
            if IMAGE_TYPES_PRIORITY.count(label) > 0:
                rank = IMAGE_TYPES_PRIORITY.index(label)
                if rank > prev_rank:
                    value = size.attrib['source']
                    prev_rank = rank
        return value

    def thumbnail(self, name):
        return self.get_size(name, 'Thumbnail')

    def get_size(self, name, image_size, resp=None):
        if resp is None:
            resp = self.flickr.photos_getSizes(photo_id=name)
            self._check_response(resp)

        for size in resp.findall('sizes/size'):
            label = size.attrib['label']
            if label == image_size:
                return size.attrib['source']
        return ""



from django import template

register = template.Library()


class IgnoreNode(template.Node):
    def __init__(self):
        pass

    def render(self, context):
        return ""


@register.tag
def ignore(parser, token):
    node_list = parser.parse(('endignore',))
    return IgnoreNode()


__author__ = 'ajumell'

from django_helpers.helpers.templatetags import TemplateNode
from django.contrib.staticfiles import finders
from django import template

JS_FILES = []
CSS_FILES = []

def add_js_file(name):
    if JS_FILES.count(name) == 0:
        JS_FILES.append(name)


def add_css_file(name):
    if CSS_FILES.count(name) == 0:
        CSS_FILES.append(name)

# TODO: Validate if files exits
# TODO: Find a method to make this to one file
# TODO: Check absolute urls in included files

def get_code(template, array):
    from django.conf import settings

    static_url = getattr(settings, 'STATIC_URL')
    code = ""
    for f in array:
        if f.startswith("http://") or f.startswith("https://"):
            s = ""
        else:
            s = static_url
            absolute_path = finders.find(f)
            if not absolute_path:
                raise Exception("Static file cannot be found.")
        code += template % (s, f)
    return code


def get_js_code():
    return get_code('<script type="text/javascript" src="%s%s"></script>', JS_FILES)

def get_css_code():
    return get_code('<link rel="stylesheet" href="%s%s" />', CSS_FILES)

register = template.Library()

class StaticFilesNode(TemplateNode):
    def render(self, context):
        return get_css_code() + get_js_code()


@register.tag
def static_files(parser, token):
    return StaticFilesNode()


def add_jquery():
    add_js_file('js/jquery-1.7.1.min.js')


def add_jquery_ui():
    add_js_file('js/jquery-ui.js')



from django.core.mail.message import EmailMultiAlternatives, EmailMessage
from django.conf import settings
from mimetypes import guess_type
from os.path import basename
from django.utils.html import strip_tags
from django_helpers.helpers.views import render_to_string

__author__ = 'ajumell'

from_email = "dummy"
if hasattr(settings, "EMAIL_HOST_USER"):
    from_email = settings.EMAIL_HOST_USER

def attach(mail, attachments):
    if attachments is not None:
        for attachment in attachments:
            file_pointer = open(attachment)
            contents = file_pointer.read()
            file_pointer.close()
            file_name = basename(attachment)
            file_type = None
            file_types = guess_type(file_name)
            for type in file_types:
                if type is not None:
                    file_type = type
                    break
            print file_name, file_type
            mail.attach(file_name, contents, file_type)


def send_text_mail(subject, contents, to, attachments=None):
    mail = EmailMessage(subject, contents, from_email, [to])
    attach(mail, attachments)
    mail.send()


def send_html_email(subject, text_content, html_content, to, attachments=None):
    mail = EmailMultiAlternatives(subject, text_content, from_email, [to])
    mail.attach_alternative(html_content, "text/html")
    attach(mail, attachments)
    mail.send()


def render_template(subject, template, to, attachments=None, template_dict=None, request=None):
    html = render_to_string(template, template_dict, request)
    text = strip_tags(html)
    send_html_email(subject, text, html, to, attachments)


from django.template.base import kwarg_re, TemplateSyntaxError, Node

__author__ = 'ajumell'

class TemplateNode(Node):
    def __init__(self, args=None, kwargs=None):
        self.args = args
        self.kwarg = kwargs

def parse_args(bits, parser):
    args = []
    kwargs = {}
    for bit in bits[1:]:
        match = kwarg_re.match(bit)
        if not match:
            raise TemplateSyntaxError("Malformed arguments to %s tag" % bits[0])
        name, value = match.groups()
        val = parser.compile_filter(value)
        if name:
            kwargs[name] = val
        else:
            if len(kwargs) == 0:
                args.append(val)
            else:
                raise TemplateSyntaxError("A normal argument cannot preceed keyword argument.")
    return args, kwargs


def create_context_from_kwargs(kwargs, new_context, names, context):
    for name in names:
        if kwargs.has_key(name):
            new_context[name] = kwargs[name].resolve(context)


from django import template
register = template.Library()

class LogNode(template.Node):
    def __init__(self, obj):
        self.obj = obj

    def render(self, context):
        obj = self.obj.resolve(context, True)
        print obj
        return ''

@register.tag
def log(parser, token):
    bits = token.contents.split()
    if len(bits) < 2:
        raise template.TemplateSyntaxError("'%s' tag at least takes one arguments" % bits[0])
    value = parser.compile_filter(bits[1])
    return LogNode(value)



class DIRNode(template.Node):
    def __init__(self, obj):
        self.obj = obj

    def render(self, context):
        obj = self.obj.resolve(context, True)
        for x in dir(obj): print x
        return ''


@register.tag
def dir(parser, token):
    bits = token.contents.split()
    if len(bits) < 2:
        raise template.TemplateSyntaxError("'%s' tag at least takes one arguments" % bits[0])
    value = parser.compile_filter(bits[1])
    return DIRNode(value)



__author__ = 'ajumell'

import tweepy

consumer_key = "2gSVq31o1icSftlDTrCRQ"
consumer_secret = "kNxG4EEHO9PG0Xvi4hgRjlQshKpqOcZdtlUM94MN8M"

token = "103590211-DHKjfZlL83WmVNb8TyfPLCZd8y9ieB6QNt6p3gRB"
secret = "mRscTV1O2H55aoL4b9pSnBup0DB3BTNGQTBho6BYks"

auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
auth.set_access_token(token, secret)

api = tweepy.API(auth)
def post_tweet():
    x = api.update_status("My first tweet from my application")
    print x

def get_public_tweets():
    public_tweets = tweepy.api.user_timeline('ajumell')
    for tweet in public_tweets:
        print tweet.text
        print tweet.author
        print tweet.user
        user = tweet.user

        print user.name
        print user.screen_name
        print user.profile_image_url

        for x in dir(tweet):
            print x

def send_direct_message():
    api.send_direct_message(screen_name="sujiths777", text="Dude my first direct message from tweet app")


get_public_tweets()


"""
This file provide database settings for clear db
database heroku addon.
Example
-------
from django_helpers.heroku.clear_db_settings import DATABASES
"""
__author__ = 'ajumell'

import database

if database.has_clear_db():
    DATABASES = {
        'default': database.parse_clear_db()
    }



from django import template
from urllib2 import unquote

register = template.Library()


class NoOopNode(template.Node):
    def __init__(self, text):
        self.text = text

    def render(self, context):
        output = ""
        for bit in self.text:
            if isinstance(bit, basestring):
                output += bit
            else:
                output += bit.render(context)
        while output.find('\n') >= 0:
            output = output.replace('\n', '')
        return unquote(output)


@register.tag
def unquote_noop(parser, token):
    text_and_nodes = []
    while 1:
        token = parser.tokens.pop(0)
        if token.contents == 'endnoop':
            break

        if token.token_type == template.TOKEN_VAR:
            text_and_nodes.append('{{')
            text_and_nodes.append(token.contents)

        elif token.token_type == template.TOKEN_TEXT:
            text_and_nodes.append(token.contents)

        elif token.token_type == template.TOKEN_BLOCK:
            try:
                command = token.contents.split()[0]
            except IndexError:
                parser.empty_block_tag(token)

            try:
                compile_func = parser.tags[command]
            except KeyError:
                parser.invalid_block_tag(token, command, None)
            try:
                node = compile_func(parser, token)
            except template.TemplateSyntaxError, e:
                if not parser.compile_function_error(token, e):
                    raise
            text_and_nodes.append(node)

        if token.token_type == template.TOKEN_VAR:
            text_and_nodes.append('}}')
    return NoOopNode(text_and_nodes)


from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.template import RequestContext, loader, Context
from django.utils.simplejson import dumps
from django.views.decorators.csrf import csrf_exempt


def redirect(to, args=None, kwargs=None):
    return HttpResponseRedirect(reverse(to, args=args, kwargs=kwargs))


def render_to_response(template, request, data=None):
    """
    Renders the given template with given data and returns the HttpResponse

    @type request: django.http.HttpRequest
    @param request: The request object

    @type data: dict
    @param data: The extra variables that has to be passed to the context

    @type template: string
    @param template: The name of the template to be rendered

    @rtype: HttpResponse
    """
    context = RequestContext(request, data)
    template = loader.get_template(template)
    html = template.render(context)
    return HttpResponse(html)


def render_template(request, template, data=None, url_params=None, **kwargs):
    if url_params is not None:
        if data is None:
            data = dict()

        for url_param in url_params:
            param_value = kwargs.get(url_param, '')
            if param_value != "":
                data[url_param] = param_value
    return render_to_response(template, request, data)


def render_json(dictionary):
    string = dumps(dictionary)
    mimetype = 'application/javascript'
    return HttpResponse(string, mimetype=mimetype)


def render_to_string(template, values=None, request=None):
    template = loader.get_template(template)
    if request is None:
        context = Context(values)
    else:
        context = RequestContext(request, values)
    html = template.render(context)
    return html


def message(request, heading, message):
    return render_to_response('generic/message.html', request, {
        'heading': heading,
        'msg': message
    })


def render_error(request, message):
    return render_to_response('generic/error.html', request, {
        'heading': "Error Occured",
        'msg': message
    })


def render_form(request, form, extra=None):
    if extra is None:
        extra = {}
    extra["form"] = form
    return render_to_response('generic/form.html', request, extra)


def render_confirm(request, message, heading=None):
    return render_to_response('generic/confirm.html', request, {
        'heading': heading,
        'msg': message
    })


def is_multipart(Form):
    """
    Returns true if the given form require multipart attributr

    @param Form: A form class

    @rtype Boolean
    """
    for x in Form.base_fields:
        y = Form.base_fields[x]
        if y.widget.needs_multipart_form:
            return True
    return False


def form_view(request,
              Form,
              success_view,
              form_heading,
              success_view_args=None,
              success_message=None,
              current_user_field=None,
              submit_button="Submit",
              error_message="Please correct the errors.",
              template="generic/form.html",
              ajax_template="partials/form.html",
              obj_instance=None,
              custom_form_arguments=None,
              template_dict=None,
              obj_url_params=None,
              has_many_to_many=False,
              url_params=None,
              **kwargs
):
    """
    Returns a HttpResponse by rendering a form with given parameters

    @param Form: The form to be rendered

    @param string success_view: The name of the view to be rendered when the
    form has successfully saved.

    @rtype: HttpResponse
    """

    if custom_form_arguments is None:
        custom_form_arguments = {}

    if obj_instance is not None:
        custom_form_arguments['instance'] = obj_instance

    if template_dict is None:
        template_dict = {}

    if request.method == "POST":
        if is_multipart(Form):
            form = Form(request.POST, request.FILES, **custom_form_arguments)
        else:
            form = Form(request.POST, **custom_form_arguments)

        if form.is_valid():
            obj = form.save(has_many_to_many)

            if current_user_field is not None:
                setattr(obj, current_user_field, request.user)

            if obj_url_params is not None:
                for param in obj_url_params:
                    if type(param) == tuple:
                        val = kwargs.get(param[1])
                        param = param[0]
                    else:
                        val = kwargs.get(param)
                    setattr(obj, param, val)

            obj.save()
            if request.is_ajax():
                response_dict = {
                    "success": True,
                    "message": success_message
                }
                return render_json(response_dict)
            else:
                if success_message is not None:
                    messages.success(request, success_message)

                if success_view_args is not None:
                    args = list()
                    for argument in success_view_args:
                        args.append(kwargs.get(argument))
                    return redirect(success_view, tuple(args))
                else:
                    return redirect(success_view)
        else:
            if request.is_ajax():
                rdict = {'error': True}
                d = {}
                for e in form.errors:
                    d.update({e: form.errors[e]})
                rdict.update({'errors': d})
                return render_json(rdict)
            template_dict['form_error_message'] = error_message
    else:
        form = Form(**custom_form_arguments)

    if url_params is not None:
        for url_param in url_params:
            param_value = kwargs.get(url_param, '')
            if param_value != "":
                template_dict[url_param] = param_value

    template_dict['form'] = form
    template_dict['form_submit'] = submit_button
    template_dict['form_heading'] = form_heading

    if request.is_ajax():
        template = ajax_template

    return render_to_response(template, request, template_dict)


@csrf_exempt
def delete_items_ajax(request, Model):
    """
    A generic view to delete items from a model using ajax.
    @param request:
    @type request: django.http.HttpRequest
    @return:
    """
    if request.is_ajax():
        values = request.POST.getlist('values[]')
        query = Model.objects.filter(pk__in=values)
        query.delete()
        return HttpResponse('true')
    return HttpResponse('false')



import re
from urllib2 import quote

from django import template
from django.utils.safestring import mark_safe
from django_helpers.helpers.social.twitter import get_public_tweets
from django_helpers.helpers.templatetags import parse_args, TemplateNode, create_context_from_kwargs
from django_helpers.helpers.views import render_to_string

register = template.Library()

__author__ = 'ajumell'

class TwitterTweetsNode(template.Node):
    def __init__(self, username):
        self.username = username

    def render(self, context):
        username = self.username.resolve(context, True)
        tweets = get_public_tweets(username)
        return render_to_string('tags/social/twitter.html',{
            'tweets': tweets
        })

@register.tag
def tweets(parser, token):
    bits = token.contents.split()
    if len(bits) < 2:
        raise template.TemplateSyntaxError("'%s' tag at least takes one arguments" % bits[0])
    value = parser.compile_filter(bits[1])
    return TwitterTweetsNode(value)


username_re = re.compile('[@]+([A-Za-z0-9_]+)')
hash_re = re.compile('[#]+([A-Za-z0-9_]+)')
link_re = re.compile(r"(http://[^ ]+)")

@register.filter
def format_tweet(value):
    value = link_re.sub(r'<a href="\1">\1</a>', value)
    value = hash_re.sub(r'<a href="http://search.twitter.com/search?q=\1">#\1</a>', value)
    value = username_re.sub(r'<a href="http://twitter.com/\1">@\1</a>', value)
    return mark_safe(value)

format_tweet.si_safe = True

class FacebookLikeButtonNode(TemplateNode):

    def render(self, context):
        args = self.args
        kwargs = self.kwarg
        cntxt = {}

        if len(args) != 0:
            url = args[1]
        elif kwargs.has_key('url'):
            url = kwargs['url']
        else:
            for c in context:
                print c
            url = context.get('current_full_url')

        if not url:
            raise template.TemplateSyntaxError("URL is no provided.")

        cntxt['url'] = quote(url)
        names = ['height', 'width', 'scheme', 'font']
        create_context_from_kwargs(kwargs, cntxt, names, context)
        return render_to_string('tags/social/facebook-like.html',cntxt)

@register.tag
def facebook_like(parser, token):
    bits = token.contents.split()
    args, kwargs = parse_args(bits, parser)
    return FacebookLikeButtonNode(args, kwargs)




from django.template.loader_tags import do_include, BaseIncludeNode
from helpers import remove_breaks
from django import template

register = template.Library()

class SpacelessIncludeNode(BaseIncludeNode):
    def __init__(self, template, *args, **kwargs):
        BaseIncludeNode.__init__(self, *args, **kwargs)
        self.template = template

    def render(self, context):
        if not self.template:
            return ''
        op = self.render_template(self.template, context)
        return remove_breaks(op)


@register.tag('spaceless_include')
def spaceless_include(parser, token):
    node = do_include(parser, token)
    args_dict = {
        "extra_context": node.extra_context,
        "isolated_context": node.isolated_context,
        "template": node.template
    }
    return SpacelessIncludeNode(**args_dict)


__author__ = 'ajumell'


def request_details(request):
    return {
        'current_path': request.get_full_path(),
        'current_domain': request.get_host(),
        'is_https': request.is_secure(),
        'is_ajax': request.is_ajax(),
        'current_full_url': "%s%s" % (request.get_host(), request.get_full_path())
    }



from django_helpers.helpers.templatetags import TemplateNode, parse_args
from django_helpers.helpers.views import render_to_string
from django_helpers.templatetags.static_manger import register, add_jquery


__author__ = 'ajumell'

class BackToTopNode(TemplateNode):

    def __init__(self, args, kwargs):
        TemplateNode.__init__(self, args, kwargs)
        add_jquery()

    def render(self, context):
        args = self.args
        kwargs = self.kwarg
        if len(args) > 0:
            template = args[0].resolve(context)
        else:
            template = 'tags/extras/back-to-top.html'

        if kwargs.has_key('position'):
            p = kwargs['position']
            p = p.resolve(context)
        else:
            p = 100

        return render_to_string(template,{
            'position': p
        })

@register.tag
def back_to_top(parser, token):
    bits = token.split_contents()
    args, kwargs = parse_args(bits, parser)
    return BackToTopNode(args, kwargs)



from django import template
from django.conf import settings
from django.template.base import TemplateSyntaxError
from django.template.defaulttags import Node

register = template.Library()
label = settings.PROJECT_LABEL

class GetAttrNode(Node):
    def __init__(self, parser, obj, name):
        Node.__init__(self)
        self.name = name
        self.obj = obj
        self.parser = parser

    def render(self, context):
        p = self.parser
        v = self.obj
        n = self.name
        tkn = p.compile_filter(n)
        n = tkn.resolve(context)
        if n:
            v = str(v) + '.' + str(n)
        v = p.compile_filter(v)
        return v.resolve(context)


class GetMatrixNode(Node):
    def __init__(self, parser, obj, x, y):
        Node.__init__(self)
        self.x = x
        self.y = y
        self.obj = obj
        self.parser = parser

    def compile(self, val, context, s=True):
        p = self.parser
        tkn = p.compile_filter(val)
        k = tkn.resolve(context)
        if s: return str(k)
        return k

    def render(self, context):
        v = self.obj
        x = self.compile(self.x, context)
        y = self.compile(self.y, context)
        v = self.compile(v + '.' + x + '.' + y, context, False)
        return v


@register.tag(name="get_attr")
def get_attr(parser, token):
    bits = token.split_contents()
    if len(bits) != 3:
        raise TemplateSyntaxError("'%s' takes exactly two arguments." % bits[0])
    obj = bits[1]
    name = bits[2]
    return GetAttrNode(parser, obj, name)


@register.tag(name="get_matrix")
def get_attr(parser, token):
    bits = token.split_contents()
    if len(bits) != 4:
        raise TemplateSyntaxError("'%s' takes exactly three arguments." % bits[0])
    obj = bits[1]
    x = bits[2]
    y = bits[3]
    return GetMatrixNode(parser, obj, x, y)



from django_helpers.helpers.views import render_to_string
from django_helpers.templatetags.static_manger import add_js_file, add_jquery

__author__ = 'ajumell'

class FormRenderer(object):
    form_submit = 'Submit'
    focus_first = None
    form_id = None
    has_validation = True
    has_rules = True
    template = 'form-renderer/form.html'
    method = 'POST'
    form_action = ''

    def __init__(self, form, request=None, form_id=None, csrf_token=None, template=None):

        add_jquery()
        add_js_file('form-renderer/js/jquery.validate.min.js')

        self.form = form
        self.request = request
        self.csrf_token = csrf_token

        if template is not None:
            self.template = template

        if self.request is None and csrf_token is None:
            raise Exception("Either request or csrf_token is needed.")

        if form_id is not None:
            self.form_id = form_id


    def __str__(self):
        return self.render()


    def __unicode__(self):
        return self.render()

    def render(self):
        if self.form_id is None:
            raise Exception("Form ID is necessary")

        return render_to_string(self.template, {
            'form': self.form,
            'form_submit': self.form_submit,
            'focus_first': self.focus_first,
            'form_id': self.form_id,
            'has_validation': self.has_validation,
            'has_rules': self.has_rules,
            'method': self.method,
            'form_action': self.form_action,
            'csrf_token_html': self.csrf_token
        }, self.request)
